function prob = pbp(U, T)

% Modified 2018 
% PBP cdf of the supremum of standardized Brownian motion
%
% Syntax
%
%   v = pbp(u, t)
%
% Description
% PNBP(U,T) computes the cumulative distribution function of the supremum
%of the normalied Brownian motion on  [1,T] at each of the
% values in U.
% The input parameter T must be a positive scalar. The values in U must
% be positive.
% The default value of the parameter T is 2.
%
% References
% 1- DeLong, David M. "Crossing probabilities for a square root boundary by
%    a Bessel process", Commun. Stat., Theory Methods A, 1981.
% 2- "One-Sided Test for an Unknown Breakpoint: Theory, Computation,
%    and Application to Monetary Theory" by Arturo Estrella & Anthony P.
%    Rodrigues Staff Report no. 232 November 2005
%

% REMARK: This function uses the program 'pu.m' by E. Cojocaru to compute
% the Cylinder Hypergeometric Function Dv(x)version
%
% 2012/10/11
% Li-Vang Lozada Chang
%
if nargin < 2,
    T = 2;
end
if T <= 1,
    disp('Error value: T must be greater than 1')
    prob = [];
    return
end

prob = 0*U;
k = 1;
for c = U,
    term = 1;
    nu = 0;
    while (abs(term)>1e-6)
        nu = zeroDw(nu,-c);
        aux1 = Dw(nu-1,-c)*npdf(c);
        aux2 = nu*derivativeDw(nu,-c);
        term = -T^(-nu/2)*aux1/aux2;
        prob(k) = prob(k) + term;
        %fprintf('term\n')           % borrar
    end
    k = k + 1;
end
%fprintf('\n')                        % borrar
end
%--------------------------------------------------------------------------
% NESTED FUNCTIONS
%--------------------------------------------------------------------------
function nu = zeroDw(nu0,x)
% Given the parameters of the parabolic cylinder function this function
% calculates the first zero bigger than nu0 of the equation
% D_v(x)=0 considering D_v(x) as a function of v.
nuPrev = nu0;
nu = nu0;
while (abs(nu-nuPrev)<1e-3)
    nu0 = nu0 + 0.5;
    %fprintf('.')                              % borrar
    nu = fzero(@(nu) Dw(nu,x), nu0);
end
%fprintf(' %2.4f ',nu)                        % borrar
end

function y = Dw(v,x)
% D_v parabolic cylinder function of Whittaker.
y = pu(-v-1/2,x);
end

function d = derivativeDw(nu,x)
% Derivative of D_v respect to v.
d = (Dw(nu+1e-8,x)-Dw(nu-1e-8,x))/2e-8;
end

function y = npdf(x)
% standard normal pdf
y = exp(-x.^2/2)/sqrt(2*pi);
end
