function v = pnbb(U, T)
% PNBB cdf of the supremum of the absolute value of the Brownian Bridge 
% 
% Syntax
%    p = pnbb(u,t)
%    p = pnbb(u)
%
% Description 
% PNBB(U,T) computes the cumulative distribution function of the supremum 
% of the absolute value of the Brownian Bridge defined on [0,T] at each of 
% the values in U. 
% The parameter T and the values in U must be positive.
% When the parameter T is not present we take T = 1.
%
% Reference:
% "Probability and Real Analysis" by Dudley, R. (2004). p. 464
% "Convergence of probability measures" by Billingsley, P. (1999) p. 103 
% "A toolbox note on the distribution of the maximum of Gaussian process" 
% by Azais, J.M. & Lozada-Chang, L.  

% Li-Vang Lozada-Chang
% Version 2012/09/26
%

m = 8;  % value to determine when truncate the infinite sum
if nargin < 2,
    T = 1;
end

v = [];
for u = U,
    K = ceil(sqrt(T*((m+1)*log(10)+log(2))./(2*u.^2))); % truncation value
    k = 1:K;
    p  = 1 + 2*cumsum((-1).^k.*exp(-2*k.^2.*u^2/T)); 
    % When u is very close to 0 the computed value could be negative. 
    % To avoid that we do as follows.
    if p(end) < 0,
        P = p(find(p > 0, 1, 'last' ));
    else
        P = p(end);
    end
    v = [v, P];
end 