function V = pnsc(U, time, omega)
% PNSC cdf of the supremum of the absolute value of the Sine-Cosine process
%
% Syntax
%
%  v = pnsc(u,t,omega)
%
% Description
% PNSC(U,T,OMEGA) computes the cumulative distribution function of the 
% supremum % of the absolute value the Sine-Cosine process with parameter 
% OMEGA on [0,T]. T must be a positive scaalar 
%
% PNSC(U,T) computes the cdf taking OMEGA = 1
%
% PNSC(U) computes the cdf taking T = 1 and OMEGA = 1.
%
% Reference:
% 1- "Level Sets and Extrema of Random Processes and Fields" by
% Jean-Marc Azais, Mario Wschebor (2009). Exercise 4.1 p.130
% and companion document of toolbox. 
% 2- "A toolbox note on the distribution of the maximum of Gaussian process" 
%    by Azais, J.M. & Lozada-Chang, L.
% 

% Version 2012/12/05 
%
if nargin < 3,
    omega = 1;
    if nargin < 2,
        time = 1;
    end
end
if  time < pi/omega
    V = 0*U;
    k = 1;
    for u = U,
        V(k) = quadgk(@(t) fun(t), 0, time*omega);
        k = k + 1;
    end
    V = 2*ncdf(U)-1 - time*omega/pi*exp(-U.^2/2)+ V/pi;
else
    V = 1 - exp(-U.^2/2);
end

% -------------------------------------------------------------
% NESTED FUNCTION
% -------------------------------------------------------------
    function y = ncdf(x)
        % standard normal cdf
        y = erfc(-x ./ sqrt(2))/2;
    end

    function y = fun(t)
        % integrand function 
        if abs(t) < 1e-10, % avoids null denominator around 0
            y = 0;
        elseif abs(pi - t) < 1e-10, % avoids null denominator around pi
            y = exp(-u^2/2);
        else
            y = exp(-u^2*(1+cos(t))./sin(t).^2);
        end
    end
end