function q = qnbb(P, T)
% QNBB inverse of CDF of the supremum of the Brownian Bridge
% 
% Syntax
%
% q = qnbb(p,t)
%
% Description
% QNBB(P,T) computes the inverse of the cumulative distribution function of 
% the supremum of the absolute value of the Brownian Bridge in the interval 
% [0,T] at each of the values in P.
% The input parameter T must be a positive scalar. The values in P must
% probability values.
% The default value of parameter T is 1.
%
% Reference 
% Reference:
% "Probability and Real Analysis" by Dudley, R. (2004). p. 464
% "Convergence of probability measures" by Billingsley, P. (1999) p. 103 
% "A toolbox note on the distribution of the maximum of Gaussian process" 
% by Azais, J.M. & Lozada-Chang, L. 

% Version 2012/10/11
% Li-Vang Lozada-Chang

if nargin < 2,
    T = 1;
end
options = optimset('TolX', 1e-5, 'Display', 'off');
q = zeros(size(P));
pk = 1;
for p = P
    q(pk) = fzero(@pnbb, [1e-5, 10*T], options);
    pk = pk + 1;
end

% NESTED FUNCTION
    function v = pnbb(u)
        K = ceil(sqrt(T*((10)*log(10)+log(2))./(2*u.^2)));
        k = 1:K;
        V  = 1 + 2*cumsum((-1).^k.*exp(-2*k.^2.*u^2/T));
        if V(end) < 0,
            v = V(find(V > 0, 1, 'last' ));
        else
            v = V(end);
        end
        v = v - p;
    end
end