function Q = qsc(P,time,omega)
% QSC inverse of CDF of the sup. of abs. val. of the Sine-Cosine process
%
% Syntax
%   q = qsc(p,t,omega)
%   q = qsc(p,t)
%   q = qsc(p)
%
% Description
% QSC(P,T,OMEGA) computes the inverse of the cumulative distribution 
% function of the Sine-Cosine process with parameter OMEGA on [0,T]. 
% T must be a positive scalar 
% The default value of T is 1.
% The default value of OMEGA is 1.
% The values in P must probability values.
%
% REFERENCE:
% "Level Sets and Extrema of Random Processes and Fields" by
% Jean-Marc Azais, Mario Wschebor (2009). Exercise 4.1 p.130
%

% Version 2010/06/09 
% Li-Vang Lozada-Chang
%            

if nargin < 3,
    omega = 1;
    if nargin < 2,
        time = 1;
    end
end
Q = 0*P; 
caso = sum(omega/pi*time>[1 2]);
options = optimset('TolX', 1e-6, 'Display', 'off');

if caso == 2,
    Q = sqrt(-2*log(1-P));   %Raleigh distribution
else
    iP = 1;
    for p = P
        switch caso
            case 0 
                u = fzero(@(u) caseZero(u),0,options);
            case 1 
                u = fzero(@(u) caseOne(u),0,options);
        end
        Q(iP) = u;
        iP = iP + 1;
    end
end
% -------------------------------------------------------------
% NESTED FUNCTION
% -------------------------------------------------------------
    function v = caseZero(u)
        %
        v = ncdf(u)-time*omega/(2*pi)*exp(-u.^2/2);   %Rice formula
        v = v - p;
    end

    function v = caseOne(u)
        %
        auxD = (time*omega-pi)/1e+3;
        auxI = quadgk(@(t) fun(u,t), pi + auxD, time*omega)...
            + auxD*exp(-pi^2);
        v = ncdf(u) +(auxD-time*omega)/(2*pi)*exp(-u.^2/2) + auxI/(2*pi);
       
      
        v = v - p;
    end

    function y = ncdf(x)
        % standard normal cdf
        y = erfc(-x ./ sqrt(2))/2;
    end
 function y = fun(u,t)   %add JMA 5/7/2018
        % integrand function 
        if abs(t-pi) < 1e-10, % avoids null denominator around pi
            y = exp(-u^2/2);
        elseif abs(2*pi - t) < 1e-10, % avoids null denominator around 2*pi
            y = 0;
        else
            y = exp(-u^2*(1-cos(t))./sin(t).^2);
        end
    end
end